#!/usr/bin/perl
# Anzeige der Terminalinformation aus Turbo-Pascal 2.x 3.x
# V. Pohlers, 26.09.2011
# 21.03.2024 V2 mit Versions-Erkennung und Anzeige der Tastatur-Codes bei TURBO

# use Data::Dumper;

$IN = $ARGV[0] || "turbo.com";
open IN;
binmode IN;


our $pos;

#######################################
# Get TP Version
#######################################

#103:	34h,12h	TPVers1
#	CDh,ABh	TPVers2, TPVers3
#	
#116	30h	TPVers1 1980
#	34h	TPVers2 1984
#	35h	TPVers3	1985
#
#126	82h,41h	TPVersion == 200 Installation area for editor commands
#	23h,41h	TPVersion == 201 Installation area for editor commands
#	A1h,42h	TPVersion == 300 Installation area for editor commands 
#	83h,42h	TPVersion == 301 Installation area for editor commands
	

seek IN, 0x0003, 0;
read IN, $TPVers, 2;

seek IN, 0x0016, 0;
read IN, $TPYear, 1;

seek IN, 0x0026, 0;
read IN, $ECmdInst, 2;

if ($TPVers eq "\x34\x12") { print "Turbo 1.0\n" }
elsif ($TPVers eq "\xCD\xAB")  { 
	if ($TPYear eq '4') { print "Turbo 2\n" }
	elsif ($TPYear eq '5') { print "Turbo 3\n" }
	else  { print "unknown Turbo version (year)\n" }
	
	if ($ECmdInst eq "\x82\x41") { print "Turbo 2.00\n" }
	elsif ($ECmdInst eq "\x23\x41") { print "Turbo 2.01\n" }
	elsif ($ECmdInst eq "\xA1\x42") { print "Turbo 3.00\n" }
	elsif ($ECmdInst eq "\x83\x42") { print "Turbo 3.01\n" }
	else { print "unknown Turbo version (ECmdInst)\n" }
} else { print "unknown Turbo version (TPVers)\n" }
	

#######################################
# Terminal Configuration
#######################################

#                    ;Screen-Installationsbereich
#                    ;----------------------------
#                    
#                    ;Terminal >>
#  0153  TermTypStr:	DEFB	20,'No Terminal selected'
#  0168  Width:		DEFB	80		;Bildschirmbreite (1-256)
#  0169  Height:	DEFB	24		;Bildschirmhoehe  (1-256)
#  016A  		DEFB	0		;wozu das?
#        
#        ;Ab hier werden die Installationsdaten von der Laufzeitbibliothek verwendet
#  016B  InitialStr:	DEFS	16
#  017B  ResetStr:	DEFS	16
#        
#        ;Cursor >>
#  018B  CursorStr:	DEFS	16		;Muster fuer Cursor-Positionierung
#  019B  BinaryAddr:	DEFB	0		;0=ASCII, 1=Binary address
#  019C  ColumnOff:	DEFB	0
#  019D  LineOff:	DEFB	0
#  019E  XPosition:	DEFB	0		;Pos. in Muster
#  019F  YPosition:	DEFB	0		;	"
#  01A0  DelayCursor:	DEFW	0		;Delay after cursor address (0-255 ms)
#        
#        ;Line >>
#  01A2  ClearScrStr:	DEFS	6		;Clear screen
#  01A8  HomeStr:	DEFS	6		;Home cursor (Dieser String ist leer,
#        					;wenn Clear screen schon Home macht)
#  01AE  InsLineStr:	DEFS	6		;Insert line
#  01B4  DelLineStr:	DEFS	6		;Delete line
#  01BA  DelayLine:	DEFW	0		;Delay after Clear, Delete and Insert (0-255 ms)
#        
#        ;Highlight >>
#  01BC  EraToEolStr:	DEFS	6		;Erase to end of line
#  01C2  HLStartStr:	DEFS	6		;Start highlighting
#  01C8  HLEndStr:	DEFS	6		;End highlighting
#  01CE  DelayHL:	DEFW	0		;Delay after EraToEol and HighLight
#                    
#  01D0  

print "\nTerminal Configuration\n";

seek IN, 0x0053, 0;
read IN, $terminal, 0x0D0-0x053; 

print length($terminal), "\n";

@erg = ();
@erg = unpack("
	C		# Stringlength
	a20		# TermTypStr
	C		# Width
	C		# Height
	C		# ???
	a16		# InitialStr
	a16		# ResetStr
	a16		# CursorStr
	C		# BinaryAddr
	C		# ColumnOff
	C		# LineOff
	C		# XPosition
	C		# YPosition
	S		# DelayCursor
	a6		# ClearScrStr
	a6		# HomeStr
	a6		# InsLineStr
	a6		# DelLineStr
	S		# DelayLine
	a6		# EraToEolStr
	a6		# HLStartStr
	a6		# HLEndStr
	S		# DelayHL
	", $terminal
);

# print Data::Dumper->Dump([\@erg], ["*erg"]);

printf "0153  TermTypStr:  %d,%s\n", $erg[0], substr($erg[1],0,$erg[0]);
printf "0168  Width:       %d\n", $erg[2];
printf "0169  Height:      %d\n", $erg[3];
printf "016A  ???:         %d\n", $erg[4];
printf "016B  InitialStr:  %s\n", tostr($erg[5]);
printf "017B  ResetStr:    %s\n", tostr($erg[6]);
printf "018B  CursorStr:   %s\n", tostr($erg[7]);
printf "019B  BinaryAddr:  %d\n", $erg[8];
printf "019C  ColumnOff:   %d\n", $erg[9];
printf "019D  LineOff:     %d\n", $erg[10];
printf "019E  XPosition:   %d\n", $erg[11];
printf "019F  YPosition:   %d\n", $erg[12];
printf "01A0  DelayCursor: %d\n", $erg[13];
printf "01A2  ClearScrStr: %s\n", tostr($erg[14]);
printf "01A8  HomeStr:     %s\n", tostr($erg[15]);
printf "01AE  InsLineStr:  %s\n", tostr($erg[16]);
printf "01B4  DelLineStr:  %s\n", tostr($erg[17]);
printf "01BA  DelayLine:   %d\n", $erg[18];
printf "01BC  EraToEolStr: %s\n", tostr($erg[19]);
printf "01C2  HLStartStr:  %s\n", tostr($erg[20]);
printf "01C8  HLEndStr:    %s\n", tostr($erg[21]);
printf "01CE  DelayHL:     %d\n", $erg[22];


#######################################
# turbo or compiled program?
#######################################

seek IN, 1, 0;			# 0100: JP Start
read IN, $Start, 2;
$Start = unpack('S', $Start);	# Address Of ProgramStart
#print $Start, "\n";

seek IN, $Start-0x100, 0;
read IN, $code, 3;

if ($code eq "\x31\x00\x01") 	# ld sp, 100h
{ 	# compiled program
	print "compiled program\n";
	close IN;
	exit;
}

print "\nEditor Configuration\n";

#######################################
# Editor Configuration
#######################################

@editcmds = ('new line', #cmd 0, can't be installed
'Character left',
'  Alternative',
'Character right',
'Word left',
'Word right',
'Line up',
'Line down',
'Scroll down',
'Scroll up',
'Page up',
'Page down',
'To left on line',
'To right on line',
'To top of page',
'To bottom of page',
'To top of file',
'To end of file',
'To begining of block',
'To end of block',
'To last cursor position',
'Insert mode on/off',
'Insert line',
'Delete line',
'Delete to end of line',
'Delete right word',
'Delete character under cursor',
'Delete left character',
'  Alternative',
'Mark block begin',
'Mark block end',
'Mark single word',
'Hide/display block',
'Copy block',
'Move block',
'Delete block',
'Read block from disk',
'Write block to disk',
'End edit',
'Tab',
'Auto tab on/off',
'Restore line',
'Find',
'Find & replace',
'Repeat last find',
'Control character prefix');


$ECmdInst = unpack('S', $ECmdInst);
#print $ECmdInst, "\n";


seek IN, $ECmdInst-0x100, 0;
read IN, $erg, 200; #Lnge des Blocks
read IN, $keys, 200; #Lnge des Blocks
$pos=0;
$posKey=0;

print "addr nr command                        keys hex   keys ctrl  default keys\n";

foreach my $i (0..45) {

	if ($i==0) { print "CURSOR MOVEMENTS:\n" }
	elsif 	($i==21) { print "INSERT & DELETE:\n" }
	elsif 	($i==29) { print "BLOCK COMMANDS:\n" }
	elsif 	($i==38) { print "MISC. EDITING COMMANDS:\n" }
	
	printf "%04X %2d %-30s %-10s %-10s %-10s\n", $ECmdInst+$pos, $i, $editcmds[$i], 
		tostr(substr($erg,$pos,10)), 
		tostrCtrl(substr($erg,$pos,10), \$pos), 
		tostrCtrl(substr($keys,$posKey,10), \$posKey);	
}

close IN;
#FINIS


sub hex2 {
	return sprintf("%02X",shift);
}	

# verndert 2. ref-Parameter 
sub tostr($\$) {
	my $in = shift; my $pos = shift;
	my $out = '';
	my $len = ord(substr($in,0,1));
	for ($i=0; $i<=$len; $i++) {
		$c = substr($in,$i,1);
		
		if ($c ge ' ' && $c lt chr(0x7f) ) {
			$out .= "'$c',";
		} else {
			$out .= hex2(ord($c)) . ',';
		}
	}
	
	$$pos += $len+1; # ref
	
	$out =~ s/,$//;
	$out;
}

# verndert 2. ref-Parameter 
sub tostrCtrl($\$) {
	my $in = shift; my $pos = shift;
	my $out = '';
	my $len = ord(substr($in,0,1));
	
	$out .= hex2($len) . ','; # Lnge
	
	for ($i=1; $i<=$len; $i++) {
		$c = substr($in,$i,1);
		
		if ($c eq "\x1B") { $out .= 'ESC,' }
		elsif ($c lt ' ' ) { $out .= '^'. chr(ord($c)+0x40) . ',' }
		elsif ($c lt "\x7f" ) { $out .= "'$c'," }
		else {	$out .= hex2(ord($c)) . ',' }
	}
	
	$$pos += $len+1; # ref
	
	$out =~ s/,$//;
	$out;
}
